@@ -986,7 +986,8 @@ static void zend_optimize(zend_op_array      *op_array,
 	/* pass 9:
 	 * - Optimize temp variables usage
 	 */
-	if (ZEND_OPTIMIZER_PASS_9 & ctx->optimization_level) {
+	if ((ZEND_OPTIMIZER_PASS_9 & ctx->optimization_level) &&
+	    !(ZEND_OPTIMIZER_PASS_7 & ctx->optimization_level)) {
 		zend_optimize_temporary_variables(op_array, ctx);
 		if (ctx->debug_level & ZEND_DUMP_AFTER_PASS_9) {
 			zend_dump_op_array(op_array, 0, "after pass 9", NULL);
@@ -1474,6 +1475,15 @@ int zend_optimize_script(zend_script *script, zend_long optimization_level, zend
 			}
 		}
 
+		if (ZEND_OPTIMIZER_PASS_9 & optimization_level) {
+			for (i = 0; i < call_graph.op_arrays_count; i++) {
+				zend_optimize_temporary_variables(call_graph.op_arrays[i], &ctx);
+				if (debug_level & ZEND_DUMP_AFTER_PASS_9) {
+					zend_dump_op_array(call_graph.op_arrays[i], 0, "after pass 9", NULL);
+				}
+			}
+		}
+
 		if (ZEND_OPTIMIZER_PASS_11 & optimization_level) {
 			for (i = 0; i < call_graph.op_arrays_count; i++) {
 				zend_optimizer_compact_literals(call_graph.op_arrays[i], &ctx);
